static char rcsid[]="$Id: formlib.c,v 1.1 92/10/20 18:52:22 mos Exp $";

/* This file is (C)opyright Mauri "mos" Sahlberg, Helsinki 1992
   See LICENSE distributed with this program to see your rights
   concerning this file.

   Author: Mauri "mos" Sahlberg
   Date  : ?.?.1992
  
   Purpose: Defines some "handy" functions for screen output and kboard input
 
   Last edit: 
   $Date: 92/10/20 18:52:22 $
   $Author: mos $

   History: 
   $Log:	formlib.c,v $
 * Revision 1.1  92/10/20  18:52:22  mos
 * Initial revision
 * 

*/ 
#include <curses.h>
#include <stdlib.h>
#include <ctype.h>

#include "ruutu.h"
#include "mform.h"

int islnum(int);

void falku()
{
  WINDOW *w;
  
  w = initscr();
  savetty();
  clear();
  cbreak();
  noecho();
  keypad(w,TRUE);
}

void floppu()
{
  resetty();
  endwin();
}

/* x = x position, y = y position, len = length, *koodi = return code
   a = attributes, al = original value
*/
char *get_string(int x, int y, int len, int *koodi, int a, char *al)
{
  int k=0;		/* Merkkien m{{r{. */
  int m=0;		/* Luettu merkki.  */
  int i=0;		/* Nykyinen positio. x-koordinaatti */
  int j=0;		/* Nykyinen positio. y-koordinaatti */
  char *s=NULL;		/* Palautettava merkkijono. */
  int v=0;		/* Apumuuttuja. */
  int pos=0;		/* Positio. */

  attrset(a);
  i = x;
  j = y;
  *koodi = 0;			/* Oletusarvoisesti RETURN. */
  s =  malloc(len + 1);		/* Varataan tila. */
  if(!s) {			/* Onnistuiko? */
    perror("formlib.c/malloc");  
     exit(0);
  }
  for(m=0;m<len;m++) mvaddch(j,i+m,' ');
  refresh();
  if(al && strlen(al) > 0) {
    mvaddstr(y,x,al); 
    refresh();
    k = strlen(al);
    pos = k;		
    sprintf(s,"%s",al);
  }
  while(1) {			 
    move(j,i+pos);
    refresh();
    m = get_char();		/* Haetaan merkki. */
    switch(m) {			
      case KEY_EXIT:
	s[k]='\0';
	*koodi = KEY_EXIT;	
	return s; 	
      case KEY_LEFT:		/* Vasemmalle, jos mahdollista. */
        pos--;
	if(pos < 0) 		/* Ennen vasenta reunaa? */
	  pos = 0;
	move(j,i+pos);		
	break;
      case KEY_RIGHT:		/* Oikealle, jos mahdollista. */
	pos++;
	if(pos > k)		/* Oikea reuna = sy|te + 1 */
	  pos = k+1;
        if(pos > len)
	  pos = len;
	if(pos > k)
	  s[++k] = ' ';
	move(j,i+pos);
	break;
      case KEY_DOWN:		/* Alas, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_DOWN;	
	return s;
      case KEY_UP:		/* Yl|s, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_UP;
	return s;
      case KEY_BACKSPACE:	/* Bacspace */
	if(!pos) {	
	  beep();
	  break;
	}
	for(v =  pos -1; v < k-1; v++) {  /* P{ivitet{{n ruutu. */
	  s[v] = s[v+1];
          mvaddch(j,i+v,s[v]);
        }
        mvaddch(j,i+v,' ');
        k--;
        pos--;
	break;
      case '\n':				/* Enter. */
	s[k]='\0';
	return s;
      case KEY_DL:				/* Delete. */
        if(!k) {			/* Onko poistettavaa? */
	  beep();
	  break;
	}
	for(v=pos;v < k-1;v++) { 
	  s[v] = s[v+1];
	  mvaddch(j,i+v,s[v]);
        }  
        mvaddch(j,i+v,' ');
        k--;
	if(pos > k) {
          s[pos] = ' ';
	  k++;
	  mvaddch(j,i+pos,' ');
	}
	break;
      default:			/* Lis{t{{n merkkej{. */
	if(pos >= len) {
	  beep();
	  break;
	}
        s[pos] = m;
        if(pos == k)
	  k++;
	mvaddch(j,i+pos,m);
	pos++;
	break;
    }
  }
}

int get_char()
{
  int m;
  int ehto=0;
  int esc=0;

  while(!ehto) {
    m = getch();
    switch(m) {
      case KEY_DOWN:
      case KEY_UP:
      case KEY_LEFT:
      case KEY_RIGHT:
        return m;
      case '\b':
      case KEY_BACKSPACE:
	return KEY_BACKSPACE;
      case '\n':
      case 13:
	return '\n';
      case 127:
      case KEY_DL:
	return KEY_DL;	
      case 27:
	esc = 1;
	break;
      case 'x':
        if(esc)
	  return KEY_EXIT;	
      default:
	if(m > 255)
	  break;
	return m;	
    }
  }
  /* NOT REACHED */        
}
  
int get_one(int y, int x, int def, int *ko, int len)
{
  int ch;
  int i=0;

  for(i=x;i<len;i++)
   mvaddch(y,i,' ');
  refresh();
  ch = def;
  move(y,x);
  if(isprint(ch) && ch < 128)
    addch(ch);
  refresh();
  ch = get_char();
  if(ch == '\n' || ch > 128 || !isprint(ch)) {
    *ko = ch;
    if(ch == '\n') 
      *ko = 0;	
    return def;
  }
  mvaddch(y,x,ch);
  refresh();
  return ch;
}

long get_integer(int x, int y, int len, int *koodi, int a, char *al)
{
  int k=0;		/* Merkkien m{{r{. */
  int m=0;		/* Luettu merkki.  */
  int i=0;		/* Nykyinen positio. x-koordinaatti */
  int j=0;		/* Nykyinen positio. y-koordinaatti */
  char *s=NULL;		/* Palautettava merkkijono. */
  int v=0;		/* Apumuuttuja. */
  int pos=0;		/* Positio. */
  long luku=0L;		/* Huh. */

  attrset(a);
  i = x;
  j = y;
  *koodi = 0;			/* Oletusarvoisesti RETURN. */
  s =  malloc(len + 1);		/* Varataan tila. */
  if(!s) {			/* Onnistuiko? */
    perror("formlib.c/malloc");  
     exit(0);
  }
  for(m=0;m<len;m++) mvaddch(j,i+m,' ');
  refresh();
  if(al && strlen(al) > 0) {
    mvaddstr(y,x,al); 
    refresh();
    k = strlen(al);
    pos = k;	
    sprintf(s,"%s",al);
  }
  while(1) {			 
    move(j,i+pos);
    refresh();
    m = get_char();		/* Haetaan merkki. */
     switch(m) {			
      case KEY_EXIT:
	s[k]='\0';
	*koodi = KEY_EXIT;	
	luku = atol(s);
	free(s);
	return luku; 	
      case KEY_LEFT:		/* Vasemmalle, jos mahdollista. */
        pos--;
	if(pos < 0) 		/* Ennen vasenta reunaa? */
	  pos = 0;
	move(j,i+pos);		
	break;
      case KEY_RIGHT:		/* Oikealle, jos mahdollista. */
	pos++;
	if(pos > k)		/* Oikea reuna = sy|te + 1 */
	  pos = k+1;
        if(pos > len)
	  pos = len;
	if(pos > k)
	  s[++k] = ' ';
	move(j,i+pos);
	break;
      case KEY_DOWN:		/* Alas, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_DOWN;	
	luku = atol(s);
	free(s);
	return luku;
      case KEY_UP:		/* Yl|s, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_UP;
        luku = atol(s);
	free(s);
	return luku;
      case KEY_BACKSPACE:	/* Bacspace */
 	if(!pos) {	
	  beep();
	  break;
	}
	for(v= pos -1; v<k-1; v++) {  /* P{ivitet{{n ruutu. */
	  s[v] = s[v+1];
          mvaddch(j,i+v,s[v]);
        }
        mvaddch(j,i+v,' ');
        k--;
        pos--;
	break;
      case '\n':				/* Enter. */
	s[k]='\0';
        luku = atol(s);
	free(s);
	return luku;
      case KEY_DL:				/* Delete. */
        if(!k) {			/* Onko poistettavaa? */
	  beep();
	  break;
	}
	for(v=pos;v < k-1;v++) { 
	  s[v] = s[v+1];
	  mvaddch(j,i+v,s[v]);
        }  
        mvaddch(j,i+v,' ');
        k--;
	if(pos > k) {
          s[pos] = ' ';
	  k++;
	  mvaddch(j,i+pos,' ');
	}
	break;
      default:			/* Lis{t{{n merkkej{. */
	if(!isdigit(m) && m !='+' && m != '-') {
	  beep();
	  break;
	}
	if(pos >= len) {
	  beep();
	  break;
	}
        s[pos] = m;
        if(pos == k)
	  k++;
	mvaddch(j,i+pos,m);
	pos++;
	break;
    }
  }
  luku = atoi(s);
  free(s);
  return luku;
}

  
double get_number(int x, int y, int len, int *koodi, int a, char *al)
{
  double luku;
  int k=0;		/* Merkkien m{{r{. */
  int m=0;		/* Luettu merkki.  */
  int i=0;		/* Nykyinen positio. x-koordinaatti */
  int j=0;		/* Nykyinen positio. y-koordinaatti */
  char *s=NULL;		/* Palautettava merkkijono. */
  int v=0;		/* Apumuuttuja. */
  int pos=0;		/* Positio. */

  attrset(a);
  i = x;
  j = y;
  for(m=0;m<len;m++) mvaddch(j,i+m,' ');
  refresh();
  *koodi = 0;			/* Oletusarvoisesti RETURN. */
  s =  malloc(len + 1);		/* Varataan tila. */
  if(!s) {			/* Onnistuiko? */
    perror("formlib.c/malloc");  
     exit(0);
  }
  if(al && strlen(al) > 0) {
    mvaddstr(y,x,al); 
    refresh();
    k = strlen(al);
    pos = k;	
    sprintf(s,"%s",al);
  }
  while(1) {			 
    move(j,i+pos);
    refresh();
    m = get_char();		/* Haetaan merkki. */
    switch(m) {			
      case KEY_EXIT:
	s[k]='\0';
	*koodi = KEY_EXIT;	
	luku = atof(s);
	return luku; 	
      case KEY_LEFT:		/* Vasemmalle, jos mahdollista. */
        pos--;
	if(pos < 0) 		/* Ennen vasenta reunaa? */
	  pos = 0;
	move(j,i+pos);		
	break;
      case KEY_RIGHT:		/* Oikealle, jos mahdollista. */
	pos++;
	if(pos > k)		/* Oikea reuna = sy|te + 1 */
	  pos = k+1;
        if(pos > len)
	  pos = len;
	if(pos > k)
	  s[++k] = ' ';
	move(j,i+pos);
	break;
      case KEY_DOWN:		/* Alas, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_DOWN;	
        luku = atof(s);
	free(s);
	return luku;
      case KEY_UP:		/* Yl|s, siis lopetetaan sy|tt{minen. */
	s[k]='\0';
	*koodi = KEY_UP;
	luku = atof(s);
	free(s);
	return luku;
      case KEY_BACKSPACE:	/* Bacspace */
	if(!pos) {	
	  beep();
	  break;
	}
	for(v= pos -1 ;v<k-1;v++) { 			/* P{ivitet{{n ruutu. */
	  s[v] = s[v+1];
          mvaddch(j,i+v,s[v]);
        }
	mvaddch(j,i+v,' ');
        k--;
        pos--;
	break;
      case '\n':				/* Enter. */
	s[k]='\0';
	luku = atof(s);
	free(s);
	return luku;
      case KEY_DL:				/* Delete. */
        if(!k) {			/* Onko poistettavaa? */
	  beep();
	  break;
	}
	for(v=pos;v < k-1;v++) { 
	  s[v] = s[v+1];
	  mvaddch(j,i+v,s[v]);
        }  
	mvaddch(j,i+v,' ');
        k--;
	if(pos > k) {
          s[pos] = ' ';
	  k++;
	  mvaddch(j,i+pos,' ');
	}
	break;
      default:			/* Lis{t{{n merkkej{. */
	if(!islnum(m)) {
	  beep();
	  break;
	}
	if(pos >= len) {
	  beep();
	  break;
	}
        s[pos] = m;
        if(pos == k)
	  k++;
	mvaddch(j,i+pos,m);
	pos++;
	break;
    }
  }
  luku = atof(s);
  free(s);	
  return luku;
}

void my_box(int y1,int x1,int y2, int x2)
{
  int i;
  int j;

  mvaddch(y1,x1,ACS_ULCORNER);
  mvaddch(y1,x2,ACS_URCORNER);
  mvaddch(y2,x1,ACS_LLCORNER);
  mvaddch(y2,x2,ACS_LRCORNER);
  
  for(i=x1+1;i<x2;i++) {
    mvaddch(y1,i,ACS_HLINE);
    mvaddch(y2,i,ACS_HLINE);
  }

  for(j=y1+1;j<y2;j++) {
    mvaddch(j,x1,ACS_VLINE);
    mvaddch(j,x2,ACS_VLINE);
  }
}

int islnum(int m)
{
  int ehto =0;

  if(isdigit(m)) ehto =1;
  if(tolower(m) == 'e' || m == '.' || m == '+' || m == '-') ehto =  1;
  return ehto;
} 
